package entity;

import java.util.ArrayList;
import main.GameComponent;

public class Unit extends Entity {
	public Building target;
	public Building start;
	public Entity targetEntity;
	private int renderX, renderY;
	
	public Unit(int col) {
		teamColor = color = col;
		targetEntity = null;
		this.health = 0xFF;
	}
	
	public Building findNearestBuilding(ArrayList<Building> list) {
		Building temp = null;
		double minDistance = GameComponent.WIDTH * GameComponent.WIDTH + GameComponent.HEIGHT * GameComponent.HEIGHT;
		for (Building b : list) {
			if (start != null) {
				if (b == start)
					continue;
			}
			if (b.getTeamColor() == this.teamColor)
				continue;
			double distanceSquared = (start.x - b.x) * (start.x - b.x) + (start.y - b.y) * (start.y - b.y);
			if (minDistance > distanceSquared) {
				if (b.getTeamColor() != this.teamColor) {
					temp = b;
					minDistance = distanceSquared;
				}
			}
		}
		target = temp;
		return target;
	}
	
	public Unit findNearestUnit(ArrayList<Unit> list) {
		Unit temp = null;
		double minDistance = GameComponent.WIDTH * GameComponent.WIDTH + GameComponent.HEIGHT * GameComponent.HEIGHT;
		for (Unit u : list) {
			if (Math.rint(this.x) == Math.rint(u.x) && Math.rint(this.y) == Math.rint(u.y))
				continue;
			if (u.getTeamColor() == this.teamColor)
				continue;
			double distanceSquared = (u.x - this.x) * (u.x - this.x) + (u.y - this.y) * (u.y - this.y);
			if (minDistance > distanceSquared) {
				if (u.getTeamColor() != this.teamColor) {
					temp = u;
					minDistance = distanceSquared;
				}
			}
		}
		return temp;
	}
	
	public void findNearestEntity(ArrayList<Building> buildings, ArrayList<Unit> units) {
		Building b = this.findNearestBuilding(buildings);
		Unit u = this.findNearestUnit(units);
		
		if (b == null && u == null) {
			targetEntity = null;
			return;
		}
		
		//Priority - HIGH:      Units first.
		if (b == null && u != null) {
			targetEntity = u;
		}
		//Priority - LOW:       Buildings last.
		if (b != null && u == null) {
			targetEntity = b;
			//b.takeDamage(this);
			//TODO: Fix this problem of not being able to kill units after attacking units.
			return;
		}
		
		if (b != null && u != null) {
			double buildingDist = (b.x - this.x) * (b.x - this.x) + (b.y - this.y) * (b.y - this.y);
			double unitDist = (u.x - this.x) * (u.x - this.x) + (u.y - this.y) * (u.y - this.y);
			targetEntity = buildingDist < unitDist ? b : u;
			//if (buildingDist < unitDist)
			//b.takeDamage(this);
		}
	}
	
	public void setSpawnBuilding(Building b) {
		start = b;
		this.x = b.x;
		this.y = b.y;
		shouldDespawn = false;
	}
	
	public void setTargetBuilding(Building b) {
		target = b;
	}
	
	public void tick() {
		move();
		attack();
		checkDespawnCondition();
	}
	
	public void render(int[] data) {
		renderX = ((int) Math.rint(this.x)) / GameComponent.SCALE;
		renderY = ((int) Math.rint(this.y)) / GameComponent.SCALE;
		int length = renderY * (GameComponent.WIDTH / GameComponent.SCALE) + renderX;
		if (length < data.length && length >= 0)
			data[length] = color;
	}
	
	private void move() {
		if (targetEntity == null || start == null)
			return;
		else {
			double dx = targetEntity.x - this.x;
			double dy = targetEntity.y - this.y;
			
			if (Math.abs(dx) <= 1.0 && Math.abs(dy) <= 1.0) {
				this.x = targetEntity.x;
				this.y = targetEntity.y;
				return;
			}
			
			double dist = Math.sqrt(dx * dx + dy * dy);
			//if (dist > 0) {
			dx /= dist;
			dy /= dist;
			//}
			this.x += (Math.rint(dx));
			this.y += (Math.rint(dy));
		}
	}
	
	public void attack() {
		if (targetEntity == null || start == null)
			return;
		if (Math.rint(this.x) == Math.rint(targetEntity.x) && Math.rint(this.y) == Math.rint(targetEntity.y)) {
			//targetEntity.takeDamage(this);
			this.takeDamage(targetEntity);
		}
	}
	
	public int getColor() {
		return color;
	}
	
	public int getTeamColor() {
		return teamColor;
	}
}
