package game;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import main.GameComponent;
import main.InputHandler;
import entity.Building;
import entity.Unit;
import game.MainMenu.MenuOption;

public class DeadGame {
	
	public enum Difficulty {
		AI_EASY(300),
		AI_NORMAL(200),
		AI_HARD(150),
		AI_HARDER(100);
		
		private int value;
		
		private Difficulty(int value) {
			this.value = value;
		}
		
		public int getValue() {
			return value;
		}
	}
	
	public InputHandler inputHandler;
	public ArrayList<Building> buildings;
	public ArrayList<Unit> units;
	public MainMenu mainMenu;
	
	private boolean isInitialized;
	private boolean menuGameStartFlag; //Used to determine if the player clicks on "Start" in the main menu.
	private boolean gameOverFlag;
	private boolean greenWins;
	private boolean fadeToBlack;
	private boolean doneFadingToBlack;
	private int timerCountdown = 0;
	private int playerCountdown = 0;
	private Difficulty difficulty;
	
	public DeadGame(GameComponent g) {
		inputHandler = new InputHandler();
		buildings = new ArrayList<Building>();
		units = new ArrayList<Unit>();
		difficulty = Difficulty.AI_EASY;
		mainMenu = new MainMenu(this);
		
		g.addMouseListener(inputHandler);
		g.addMouseMotionListener(inputHandler);
		
		//		menuGameStartFlag = false;
		//		gameOverFlag = false;
		//		greenWins = false; //Default: Red always win.
		//		fadeToBlack = false; //Doesn't change value until losing condition is met.
		//		doneFadingToBlack = false; //Doesn't change value until finished fading to black.
		
		isInitialized = false;
		initialize();
	}
	
	public void tick() {
		if (menuGameStartFlag) {
			if (!isInitialized)
				initialize();
			if (!gameOverFlag) {
				if (playerCountdown > 0)
					playerCountdown--;
				if (inputHandler.mouseClicked) {
					inputHandler.mouseClicked = false;
					if (playerCountdown <= 0) {
						boolean hasBuilding = false;
						for (Building b : buildings) {
							if (inputHandler.mouseX == b.x && inputHandler.mouseY == b.y) {
								hasBuilding = true;
								break;
							}
						}
						if (hasBuilding == false) {
							Building b = new Building();
							b.x = inputHandler.mouseX;
							b.y = inputHandler.mouseY;
							b.setColor(0xFF00FF00);
							buildings.add(b);
							playerCountdown = 100;
						}
					}
				}
				for (Iterator<Building> i = buildings.iterator(); i.hasNext();) {
					Building b = i.next();
					b.tick();
					if (b.shouldDespawn())
						i.remove();
					if (b.isSendingUnitOut()) {
						//						Unit unit = new Unit(b.getColor());
						//						unit.setSpawnBuilding(b);
						//						//unit.findNearestBuilding(buildings);
						units.add(b.createUnit());
						b.finishedSendingUnitOut();
					}
				}
				for (Iterator<Unit> i = units.iterator(); i.hasNext();) {
					Unit u = i.next();
					u.tick();
					if (u.shouldDespawn()) {
						i.remove();
					}
					u.findNearestEntity(buildings, units);
				}
				randomEnemyBuilding();
				losingCondition();
			}
			if (doneFadingToBlack && gameOverFlag) {
				returnToMenu();
				//initialize();
			}
		}
		else {
			mainMenu.tick();
		}
	}
	
	public void render(int[] data) {
		if (menuGameStartFlag) {
			if (!gameOverFlag) {
				clear(data);
				renderBuildings(data);
				renderUnits(data);
				renderMouseCursor(data);
			}
			else {
				fade(data);
			}
		}
		else {
			clear(data);
			mainMenu.render(data);
		}
	}
	
	public void setComputerDifficulty(Difficulty diff) {
		this.difficulty = diff;
	}
	
	public void setGameMenuFlag(MenuOption options) {
		switch (options) {
			case START:
				menuGameStartFlag = true;
				break;
		}
	}
	
	private void renderMouseCursor(int[] data) {
		int x = inputHandler.mouseX / GameComponent.SCALE;
		int y = inputHandler.mouseY / GameComponent.SCALE;
		int length = y * (GameComponent.WIDTH / GameComponent.SCALE) + x;
		if (length < data.length && length >= 0)
			data[length] = -1;
	}
	
	private void renderBuildings(int[] data) {
		for (Building b : buildings) {
			b.render(data);
		}
	}
	
	private void renderUnits(int[] data) {
		for (Unit u : units)
			u.render(data);
	}
	
	private void initialize() {
		
		buildings.clear();
		units.clear();
		gameOverFlag = false;
		greenWins = false; //Default: Red always win.
		fadeToBlack = false; //Doesn't change value until losing condition is met.
		doneFadingToBlack = false; //Doesn't change value until finished fading to black.
		
		Random random = new Random();
		int green = random.nextInt(3) + 1;
		int red = random.nextInt(3) + 1;
		
		for (int g = 0; g < green; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (c.x == b.x && c.y == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFF00FF00);
				buildings.add(b);
			}
			else
				g--;
		}
		for (int g = 0; g < red; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (c.x == b.x && c.y == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFFFF0000);
				buildings.add(b);
			}
			else
				g--;
		}
		playerCountdown = 0;
		timerCountdown = 0;
		isInitialized = true;
	}
	
	public void uninitialize() {
		isInitialized = false;
	}
	
	public void returnToMenu() {
		menuGameStartFlag = false;
	}
	
	private void losingCondition() {
		int red = 0;
		int green = 0;
		for (Building b : buildings) {
			if (b.getTeamColor() == 0xFFFF0000)
				red++;
			if (b.getTeamColor() == 0xFF00FF00)
				green++;
		}
		if (red == 0 || green == 0) {
			//Game over.
			gameOverFlag = true;
		}
		if (red == 0)
			greenWins = true;
	}
	
	private void randomEnemyBuilding() {
		timerCountdown--;
		if (timerCountdown <= 0) {
			timerCountdown = difficulty.getValue();
			Random r = new Random();
			boolean hasBuilding = false;
			int x, y;
			do {
				x = r.nextInt(GameComponent.WIDTH);
				y = r.nextInt(GameComponent.HEIGHT);
				for (Building b : buildings) {
					if (x == b.x && y == b.y) {
						hasBuilding = true;
						break;
					}
				}
			}
			while (hasBuilding);
			Building b = new Building();
			b.x = x;
			b.y = y;
			b.setColor(0xFFFF0000);
			buildings.add(b);
		}
	}
	
	private void fade(int[] data) {
		if (!fadeToBlack) {
			if (greenWins) {
				if (checkFadingColor(data, 0xFF00FF00))
					fadeToBlack = true;
			}
			else {
				if (checkFadingColor(data, 0xFFFF0000))
					fadeToBlack = true;
			}
		}
		else {
			if (checkFadingColor(data, 0xFF000000))
				doneFadingToBlack = true;
		}
	}
	
	public void clear(int[] data) {
		for (int i = 0; i < data.length; i++)
			data[i] = 0;
	}
	
	private boolean checkFadingColor(int[] data, int valueToCheck) {
		boolean allSet = true;
		int temp = 0;
		int check = 0;
		for (int i = 0; i < data.length; i++) {
			if (data[i] != valueToCheck) {
				for (int bit = 24; bit >= 0; bit -= 8) {
					if (bit == 24) {
						temp = (data[i] & (0xFF << bit)) >> bit;
						if (temp == -1) {
							temp = 0xFF;
							data[i] = (data[i] & ~(0xFF << bit)) | (temp << bit);
						}
						else if (temp == 0xFF) {
							continue;
						}
						else {
							check = (valueToCheck & (0xFF << bit)) >> bit;
							if (temp != check) {
								if (check == -1) {
									check = 0xFF;
								}
								if (temp < check)
									temp++;
								else if (temp > check)
									temp--;
								data[i] = (data[i] & ~(0xFF << bit)) | (temp << bit);
							}
						}
						
					}
					else {
						check = (valueToCheck & (0xFF << bit)) >> bit;
						temp = (data[i] & (0xFF << bit)) >> bit;
						if (temp != check) {
							if (temp < check)
								temp++;
							else if (temp > check)
								temp--;
							data[i] = (data[i] & ~(0xFF << bit)) | (temp << bit);
						}
					}
				}
				allSet = false;
			}
		}
		return allSet;
	}
}
